from .normalize import normalize
import scipy.sparse as sps
from .hmmForwardBackward import hmmForwardBackward
import numpy as np

def hmmEM(x, init, d=None):
    # EM algorithm to fit the parameters of HMM model (a.k.a Baum-Welch algorithm)
    # Input:
    #   x: 1 x n integer vector which is the sequence of observations
    #   init: model or k
    # Output:s
    #   model: trained model structure
    #   llh: loglikelihood
    # Written by Mo Chen (sth4nth@gmail.com).
    
    if isinstance(init, tuple):
        A = init["A"]
        E = init["E"]
        s = init["s"]
        d = init["d"]
    else:  # random init with latent k
        k = init
        A = normalize(np.random.rand((k,k)), 1)
        E = normalize(np.random.rand((k,d)), 1)
        s = normalize(np.random.rand((k,1)), 0)
    n = x.shape[1]
    
    def oneHot(x, m):
        res = np.zeros((m, x.shape[0]))
        for i in range(x.shape[0]):
            res[int(x[i]), i] = 1;
        return res
    
    X = oneHot(x[0], d)
    M = np.dot(E, X)
    tol = 1e-4
    maxIter = 2
    llh = -np.inf * np.ones(maxIter)
    for iterV in range(maxIter):
        # E-step
        gamma, alpha, beta, c = hmmForwardBackward(M, A, s)
        llh[iterV] = np.sum(np.log(c[c > 0]))
        # check likelihood for convergence
        if llh[iterV] - llh[iterV - 1] < tol * np.abs(llh[iterV - 1]):
            break
        # M-step 
        
        ######################
        # YOUR CODE GOES HERE
        ######################
                                          
    llh = llh[1:iterV]
    model = {}
    model["A"] = A;
    model["E"] = E;
    model["s"] = s;
    model["d"] = d;
    return model, llh 
